# -*- coding: utf-8 -*-

"""
Discords.com API Wrapper
~~~~~~~~~~~~~~~~~~~

A basic wrapper for the Discords.com API.

:copyright: (c) 2021 Judev1
:license: MIT, see LICENSE for more details.
"""

__title__ = "discordspy"
__author__ = "Judev1"
__license__ = "MIT"
__copyright__ = "Copyright 2021 Judev1"
__version__ = "0.1.1"

from .client import Client
from .utils import Post
